@@ -4,7 +4,7 @@ from tf_geometric.nn.conv.gcn import gcn_norm_edge, gcn_mapper
 from tf_geometric.nn.kernel.map_reduce import aggregate_neighbors, sum_reducer, identity_updater
 
 
-def sgc(x, edge_index, edge_weight, K, kernel, bias=None, renorm=True, improved=False, cache=None):
+def sgc(x, edge_index, edge_weight, K, kernel, bias=None, activation=None, renorm=True, improved=False, cache=None):
     """
     Functional API for Simple Graph Convolution (SGC).
 
@@ -14,6 +14,7 @@ def sgc(x, edge_index, edge_weight, K, kernel, bias=None, renorm=True, improved=
     :param K: Number of hops.(default: :obj:`1`)
     :param kernel: Tensor, shape: [num_features, num_output_features], weight.
     :param bias: Tensor, shape: [num_output_features], bias.
+    :param activation: Activation function to use.
     :param renorm: Whether use renormalization trick (https://arxiv.org/pdf/1609.02907.pdf).
     :param improved: Whether use improved GCN or not.
     :param cache: A dict for caching A' for GCN. Different graph should not share the same cache dict.
@@ -37,5 +38,9 @@ def sgc(x, edge_index, edge_weight, K, kernel, bias=None, renorm=True, improved=
 
     if bias is not None:
         h += bias
+
+    if activation is not None:
+        h = activation(h)
+
     return h
 
